LOGGING_FILE = "logging.ini"
LOG_FILE_PATH_CLI = "cli_secret_scanner.log"
LOG_FILE_PATH = "secret_scanner.log"

LEAKS_FOUND_EXIT_CODE = 42
NO_LEAKS_FOUND_EXIT_CODE = 0

# These values are used as vcs names whenever a CLI scan is done
CLI_VCS_LOCAL_SCAN = "LOCAL_SCAN"
CLI_VCS_BITBUCKET = "CLI_BITBUCKET"
CLI_VCS_AZURE = "CLI_AZURE"
